@@ -7,9 +7,9 @@
 
 #include <kio/job.h>
 
-#include <QTest>
 #include <QSignalSpy>
 #include <QStandardPaths>
+#include <QTest>
 
 #include "httpserver_p.h"
 
@@ -73,7 +73,7 @@ void HTTPJobTest::testErrorPage()
     job = KIO::storedGet(QUrl(server.endPoint()));
     job->setUiDelegate(nullptr);
 #if KIOCORE_BUILD_DEPRECATED_SINCE(5, 78)
-    QSignalSpy mimeTypeSpy(job, SIGNAL(mimetype(KIO::Job*,QString)));
+    QSignalSpy mimeTypeSpy(job, QOverload<KIO::Job *, const QString &>::of(&KIO::TransferJob::mimetype));
 #endif
     QSignalSpy mimeTypeFoundSpy(job, &KIO::TransferJob::mimeTypeFound);
     QVERIFY(job->exec());
@@ -94,7 +94,7 @@ void HTTPJobTest::testMimeTypeDetermination()
     KIO::StoredTransferJob *job = KIO::storedGet(QUrl(server.endPoint() + '/'));
     job->setUiDelegate(nullptr);
 #if KIOCORE_BUILD_DEPRECATED_SINCE(5, 78)
-    QSignalSpy mimeTypeSpy(job, SIGNAL(mimetype(KIO::Job*,QString)));
+    QSignalSpy mimeTypeSpy(job, QOverload<KIO::Job *, const QString &>::of(&KIO::TransferJob::mimetype));
 #endif
     QSignalSpy mimeTypeFoundSpy(job, &KIO::TransferJob::mimeTypeFound);
     QVERIFY(job->exec());
