@@ -1,30 +1,18 @@
-/* This file is part of the KDE project
-   Copyright (C) 2005 - 2007 Till Adam <adam@kde.org>
+/*
+    This file is part of the KDE project
+    SPDX-FileCopyrightText: 2005-2007 Till Adam <adam@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KACL_H
 #define KACL_H
 
-#include <qplatformdefs.h>
 #include "kiocore_export.h"
+#include <qplatformdefs.h>
 
-#include <QtCore/QPair>
-#include <QtCore/QList>
+#include <QList>
+#include <QPair>
 
 typedef QPair<QString, unsigned short> ACLUserPermissions;
 typedef QList<ACLUserPermissions> ACLUserPermissionsList;
@@ -168,7 +156,7 @@ public:
     bool setNamedGroupPermissions(const QString &name, unsigned short);
 
     /** Returns the list of all group permission entries. Each entry consists
-     * of a name/permissions pair. This is a QPair, therefor access is provided
+     * of a name/permissions pair. This is a QPair, therefore access is provided
      * via the .first and .next members.
      * @return the list of all group permission entries. */
 
@@ -192,14 +180,15 @@ public:
 
 protected:
     virtual void virtual_hook(int id, void *data);
+
 private:
     class KACLPrivate;
     KACLPrivate *const d;
-    KIOCORE_EXPORT friend QDataStream &operator<< (QDataStream &s, const KACL &a);
-    KIOCORE_EXPORT friend QDataStream &operator>> (QDataStream &s, KACL &a);
+    KIOCORE_EXPORT friend QDataStream &operator<<(QDataStream &s, const KACL &a);
+    KIOCORE_EXPORT friend QDataStream &operator>>(QDataStream &s, KACL &a);
 };
 
-KIOCORE_EXPORT QDataStream &operator<< (QDataStream &s, const KACL &a);
-KIOCORE_EXPORT QDataStream &operator>> (QDataStream &s, KACL &a);
+KIOCORE_EXPORT QDataStream &operator<<(QDataStream &s, const KACL &a);
+KIOCORE_EXPORT QDataStream &operator>>(QDataStream &s, KACL &a);
 
 #endif
