@@ -7,38 +7,22 @@
 */
 
 #include "kmountpoint.h"
-
 #include <stdlib.h>
 
 #include <config-kmountpoint.h>
 
 #include <QDir>
 #include <QFile>
-#include <QTextStream>
 #include <QFileInfo>
+#include <QTextStream>
 
 #ifdef Q_OS_WIN
 #include <qt_windows.h>
-#endif
-
-#ifdef Q_OS_WIN
 static const Qt::CaseSensitivity cs = Qt::CaseInsensitive;
 #else
 static const Qt::CaseSensitivity cs = Qt::CaseSensitive;
 #endif
 
-#if HAVE_VOLMGT
-#include <volmgt.h>
-#endif
-#if HAVE_SYS_MNTTAB_H
-#include <sys/mnttab.h>
-#endif
-#if HAVE_MNTENT_H
-#include <mntent.h>
-#elif HAVE_SYS_MNTENT_H
-#include <sys/mntent.h>
-#endif
-
 // This is the *BSD branch
 #if HAVE_SYS_MOUNT_H
 #if HAVE_SYS_TYPES_H
@@ -47,40 +31,29 @@ static const Qt::CaseSensitivity cs = Qt::CaseSensitive;
 #if HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
+// FreeBSD has a table of names of mount-options in mount.h, which is only 
+// defined (as MNTOPT_NAMES) if _WANT_MNTOPTNAMES is defined.
+#define _WANT_MNTOPTNAMES
 #include <sys/mount.h>
+#undef _WANT_MNTOPTNAMES
 #endif
 
 #if HAVE_FSTAB_H
 #include <fstab.h>
 #endif
 
-#if ! HAVE_GETMNTINFO
-# ifdef _PATH_MOUNTED
-// On some Linux, MNTTAB points to /etc/fstab !
-#  undef MNTTAB
-#  define MNTTAB _PATH_MOUNTED
-# else
-#  ifndef MNTTAB
-#   ifdef MTAB_FILE
-#    define MNTTAB MTAB_FILE
-#   else
-#    define MNTTAB "/etc/mnttab"
-#   endif
-#  endif
-# endif
+// Linux
+#if HAVE_LIBS_MOUNT_AND_BLKID
+#include <libmount/libmount.h>
+#include <blkid/blkid.h>
 #endif
 
-#ifdef _OS_SOLARIS_
-#define FSTAB "/etc/vfstab"
-#else
-#define FSTAB "/etc/fstab"
-#endif
-
-class Q_DECL_HIDDEN KMountPoint::Private
+class KMountPointPrivate
 {
 public:
-    void finalizePossibleMountPoint(DetailsNeededFlags infoNeeded);
-    void finalizeCurrentMountPoint(DetailsNeededFlags infoNeeded);
+    void resolveGvfsMountPoints(KMountPoint::List &result);
+    void finalizePossibleMountPoint(KMountPoint::DetailsNeededFlags infoNeeded);
+    void finalizeCurrentMountPoint(KMountPoint::DetailsNeededFlags infoNeeded);
 
     QString m_mountedFrom;
     QString m_device; // Only available when the NeedRealDeviceName flag was set.
@@ -90,44 +63,54 @@ public:
 };
 
 KMountPoint::KMountPoint()
-    : d(new Private)
+    : d(new KMountPointPrivate)
 {
 }
 
-KMountPoint::~KMountPoint()
+KMountPoint::~KMountPoint() = default;
+
+#if HAVE_GETMNTINFO
+
+#ifdef MNTOPT_NAMES
+static struct mntoptnames bsdOptionNames[] = {
+    MNTOPT_NAMES
+};
+
+/** @brief Get mount options from @p flags and puts human-readable version in @p list
+ * 
+ * Appends all positive options found in @p flags to the @p list
+ * This is roughly paraphrased from FreeBSD's mount.c, prmount().
+ */
+static void translateMountOptions(QStringList &list, uint64_t flags)
 {
-    delete d;
+    const struct mntoptnames* optionInfo = bsdOptionNames;
+
+    // Not all 64 bits are useful option names
+    flags = flags & MNT_VISFLAGMASK;
+    // Chew up options as long as we're in the table and there
+    // are any flags left. 
+    for (; flags != 0 && optionInfo->o_opt != 0; ++optionInfo) {
+        if (flags & optionInfo->o_opt) {
+            list.append(QString::fromLatin1(optionInfo->o_name));
+            flags &= ~optionInfo->o_opt;
+        }
+    }
 }
-
-// There are (at least) four kind of APIs:
-// setmntent + getmntent + struct mntent (linux...)
-//             getmntent + struct mnttab
-// getmntinfo + struct statfs&flags (BSD 4.4 and friends)
-// getfsent + char* (BSD 4.3 and friends)
-
-#if HAVE_SETMNTENT
-#define SETMNTENT setmntent
-#define ENDMNTENT endmntent
-#define STRUCT_MNTENT struct mntent *
-#define STRUCT_SETMNTENT FILE *
-#define GETMNTENT(file, var) ((var = getmntent(file)) != nullptr)
-#define MOUNTPOINT(var) var->mnt_dir
-#define MOUNTTYPE(var) var->mnt_type
-#define MOUNTOPTIONS(var) var->mnt_opts
-#define FSNAME(var) var->mnt_fsname
 #else
-#define SETMNTENT fopen
-#define ENDMNTENT fclose
-#define STRUCT_MNTENT struct mnttab
-#define STRUCT_SETMNTENT FILE *
-#define GETMNTENT(file, var) (getmntent(file, &var) == nullptr)
-#define MOUNTPOINT(var) var.mnt_mountp
-#define MOUNTTYPE(var) var.mnt_fstype
-#define MOUNTOPTIONS(var) var.mnt_mntopts
-#define FSNAME(var) var.mnt_special
+/** @brief Get mount options from @p flags and puts human-readable version in @p list
+ * 
+ * This default version just puts the hex representation of @p flags
+ * in the list, because there is no human-readable version.
+ */
+static void translateMountOptions(QStringList &list, uint64_t flags)
+{
+    list.append(QStringLiteral("0x%1").arg(QString::number(flags, 16)));
+}
 #endif
 
-void KMountPoint::Private::finalizePossibleMountPoint(DetailsNeededFlags infoNeeded)
+#endif // HAVE_GETMNTINFO
+
+void KMountPointPrivate::finalizePossibleMountPoint(KMountPoint::DetailsNeededFlags infoNeeded)
 {
     if (m_mountedFrom.startsWith(QLatin1String("UUID="))) {
         const QStringRef uuid = m_mountedFrom.midRef(5);
@@ -144,7 +127,7 @@ void KMountPoint::Private::finalizePossibleMountPoint(DetailsNeededFlags infoNee
         }
     }
 
-    if (infoNeeded & NeedRealDeviceName) {
+    if (infoNeeded & KMountPoint::NeedRealDeviceName) {
         if (m_mountedFrom.startsWith(QLatin1Char('/'))) {
             m_device = QFileInfo(m_mountedFrom).canonicalFilePath();
         }
@@ -156,9 +139,9 @@ void KMountPoint::Private::finalizePossibleMountPoint(DetailsNeededFlags infoNee
     }
 }
 
-void KMountPoint::Private::finalizeCurrentMountPoint(DetailsNeededFlags infoNeeded)
+void KMountPointPrivate::finalizeCurrentMountPoint(KMountPoint::DetailsNeededFlags infoNeeded)
 {
-    if (infoNeeded & NeedRealDeviceName) {
+    if (infoNeeded & KMountPoint::NeedRealDeviceName) {
         if (m_mountedFrom.startsWith(QLatin1Char('/'))) {
             m_device = QFileInfo(m_mountedFrom).canonicalFilePath();
         }
@@ -167,41 +150,59 @@ void KMountPoint::Private::finalizeCurrentMountPoint(DetailsNeededFlags infoNeed
 
 KMountPoint::List KMountPoint::possibleMountPoints(DetailsNeededFlags infoNeeded)
 {
-#ifdef Q_OS_WIN
-    return KMountPoint::currentMountPoints(infoNeeded);
-#endif
-
     KMountPoint::List result;
 
-#if HAVE_SETMNTENT
-    STRUCT_SETMNTENT fstab;
-    if ((fstab = SETMNTENT(FSTAB, "r")) == nullptr) {
-        return result;
-    }
+#ifdef Q_OS_WIN
+    result = KMountPoint::currentMountPoints(infoNeeded);
 
-    STRUCT_MNTENT fe;
-    while (GETMNTENT(fstab, fe)) {
-        if (QByteArray(MOUNTTYPE(fe)) == "swap") {
-            continue;
-        }
-        Ptr mp(new KMountPoint);
-        mp->d->m_mountedFrom = QFile::decodeName(FSNAME(fe));
+#elif HAVE_LIBS_MOUNT_AND_BLKID
+    if (struct libmnt_table *table = mnt_new_table()) {
+        // By default parses "/etc/fstab"
+        if (mnt_table_parse_fstab(table, nullptr) == 0) {
+            struct libmnt_iter *itr = mnt_new_iter(MNT_ITER_FORWARD);
+            struct libmnt_fs *fs;
 
-        mp->d->m_mountPoint = QFile::decodeName(MOUNTPOINT(fe));
-        mp->d->m_mountType = QFile::decodeName(MOUNTTYPE(fe));
+            while (mnt_table_next_fs(table, itr, &fs) == 0) {
+                const char *fsType = mnt_fs_get_fstype(fs);
+                if (qstrcmp(fsType, "swap") == 0) {
+                    continue;
+                }
 
-        if (infoNeeded & NeedMountOptions) {
-            QString options = QFile::decodeName(MOUNTOPTIONS(fe));
-            mp->d->m_mountOptions = options.split(QLatin1Char(','));
+                Ptr mp(new KMountPoint);
+                mp->d->m_mountType = QFile::decodeName(fsType);
+                mp->d->m_mountPoint = QFile::decodeName(mnt_fs_get_target(fs));
+
+                // First field in /etc/fstab, e.g. /dev/sdXY, LABEL=, UUID=, /some/bind/mount/dir
+                // or some network mount
+                if (const char *source = mnt_fs_get_source(fs)) {
+                    mp->d->m_mountedFrom = QFile::decodeName(source);
+                    if (mp->d->m_mountedFrom.startsWith(QLatin1String("UUID")) || mp->d->m_mountedFrom.startsWith(QLatin1String("LABEL"))) {
+                        // Use blkid to resolve UUID/LABEL to the device file
+                        if (char *blkSource = blkid_evaluate_spec(source, nullptr)) {
+                            mp->d->m_mountedFrom = QFile::decodeName(blkSource);
+                            free(blkSource);
+                            if ((infoNeeded & KMountPoint::NeedRealDeviceName) //
+                                && mp->d->m_mountedFrom.startsWith(QLatin1String("/dev/"))) {
+                                mp->d->m_device = mp->d->m_mountedFrom;
+                            }
+                        }
+                    }
+                }
+
+                if (infoNeeded & NeedMountOptions) {
+                    mp->d->m_mountOptions = QFile::decodeName(mnt_fs_get_options(fs)).split(QLatin1Char(','));
+                }
+
+                result.append(mp);
+            }
+            mnt_free_iter(itr);
         }
 
-        mp->d->finalizePossibleMountPoint(infoNeeded);
-
-        result.append(mp);
+        mnt_free_table(table);
     }
-    ENDMNTENT(fstab);
-#else
-    QFile f(QLatin1String(FSTAB));
+#elif HAVE_FSTAB_H
+
+    QFile f{QLatin1String(FSTAB)};
     if (!f.open(QIODevice::ReadOnly)) {
         return result;
     }
@@ -209,7 +210,7 @@ KMountPoint::List KMountPoint::possibleMountPoints(DetailsNeededFlags infoNeeded
     QTextStream t(&f);
     QString s;
 
-    while (! t.atEnd()) {
+    while (!t.atEnd()) {
         s = t.readLine().simplified();
         if (s.isEmpty() || (s[0] == QLatin1Char('#'))) {
             continue;
@@ -218,24 +219,14 @@ KMountPoint::List KMountPoint::possibleMountPoints(DetailsNeededFlags infoNeeded
         // not empty or commented out by '#'
         const QStringList item = s.split(QLatin1Char(' '));
 
-#ifdef _OS_SOLARIS_
-        if (item.count() < 5) {
-            continue;
-        }
-#else
         if (item.count() < 4) {
             continue;
         }
-#endif
 
         Ptr mp(new KMountPoint);
 
         int i = 0;
         mp->d->m_mountedFrom = item[i++];
-#ifdef _OS_SOLARIS_
-        //device to fsck
-        i++;
-#endif
         mp->d->m_mountPoint = item[i++];
         mp->d->m_mountType = item[i++];
         if (mp->d->m_mountType == QLatin1String("swap")) {
@@ -250,13 +241,34 @@ KMountPoint::List KMountPoint::possibleMountPoints(DetailsNeededFlags infoNeeded
         mp->d->finalizePossibleMountPoint(infoNeeded);
 
         result.append(mp);
-    } //while
+    } // while
 
     f.close();
 #endif
+
     return result;
 }
 
+void KMountPointPrivate::resolveGvfsMountPoints(KMountPoint::List &result)
+{
+    if (m_mountedFrom == QLatin1String("gvfsd-fuse")) {
+        const QDir gvfsDir(m_mountPoint);
+        const QStringList mountDirs = gvfsDir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
+        for (const QString &mountDir : mountDirs) {
+            const QString type = mountDir.section(QLatin1Char(':'), 0, 0);
+            if (type.isEmpty()) {
+                continue;
+            }
+
+            KMountPoint::Ptr gvfsmp(new KMountPoint);
+            gvfsmp->d->m_mountedFrom = m_mountedFrom;
+            gvfsmp->d->m_mountPoint = m_mountPoint + QLatin1Char('/') + mountDir;
+            gvfsmp->d->m_mountType = type;
+            result.append(gvfsmp);
+        }
+    }
+}
+
 KMountPoint::List KMountPoint::currentMountPoints(DetailsNeededFlags infoNeeded)
 {
     KMountPoint::List result;
@@ -275,12 +287,7 @@ KMountPoint::List KMountPoint::currentMountPoints(DetailsNeededFlags infoNeeded)
         Ptr mp(new KMountPoint);
         mp->d->m_mountedFrom = QFile::decodeName(mounted[i].f_mntfromname);
         mp->d->m_mountPoint = QFile::decodeName(mounted[i].f_mntonname);
-
-#ifdef __osf__
-        mp->d->m_mountType = QFile::decodeName(mnt_names[mounted[i].f_type]);
-#else
         mp->d->m_mountType = QFile::decodeName(mounted[i].f_fstypename);
-#endif
 
         if (infoNeeded & NeedMountOptions) {
             struct fstab *ft = getfsfile(mounted[i].f_mntonname);
@@ -288,7 +295,7 @@ KMountPoint::List KMountPoint::currentMountPoints(DetailsNeededFlags infoNeeded)
                 QString options = QFile::decodeName(ft->fs_mntops);
                 mp->d->m_mountOptions = options.split(QLatin1Char(','));
             } else {
-                // TODO: get mount options if not mounted via fstab, see mounted[i].f_flags
+                translateMountOptions(mp->d->m_mountOptions, mounted[i].f_flags);
             }
         }
 
@@ -298,7 +305,7 @@ KMountPoint::List KMountPoint::currentMountPoints(DetailsNeededFlags infoNeeded)
     }
 
 #elif defined(Q_OS_WIN)
-    //nothing fancy with infoNeeded but it gets the job done
+    // nothing fancy with infoNeeded but it gets the job done
     DWORD bits = GetLogicalDrives();
     if (!bits) {
         return result;
@@ -312,49 +319,42 @@ KMountPoint::List KMountPoint::currentMountPoints(DetailsNeededFlags infoNeeded)
         }
     }
 
-#elif !defined(Q_OS_ANDROID)
-    STRUCT_SETMNTENT mnttab;
-    if ((mnttab = SETMNTENT(MNTTAB, "r")) == nullptr) {
-        return result;
-    }
+#elif HAVE_LIBS_MOUNT_AND_BLKID
+    if (struct libmnt_table *table = mnt_new_table()) {
+        // By default, parses "/proc/self/mountinfo"
+        if (mnt_table_parse_mtab(table, nullptr) == 0) {
+            struct libmnt_iter *itr = mnt_new_iter(MNT_ITER_FORWARD);
+            struct libmnt_fs *fs;
 
-    STRUCT_MNTENT fe;
-    while (GETMNTENT(mnttab, fe)) {
-        Ptr mp(new KMountPoint);
-        mp->d->m_mountedFrom = QFile::decodeName(FSNAME(fe));
+            while (mnt_table_next_fs(table, itr, &fs) == 0) {
+                Ptr mp(new KMountPoint);
+                mp->d->m_mountedFrom = QFile::decodeName(mnt_fs_get_source(fs));
+                mp->d->m_mountPoint = QFile::decodeName(mnt_fs_get_target(fs));
+                mp->d->m_mountType = QFile::decodeName(mnt_fs_get_fstype(fs));
 
-        mp->d->m_mountPoint = QFile::decodeName(MOUNTPOINT(fe));
-        mp->d->m_mountType = QFile::decodeName(MOUNTTYPE(fe));
-
-        if (infoNeeded & NeedMountOptions) {
-            QString options = QFile::decodeName(MOUNTOPTIONS(fe));
-            mp->d->m_mountOptions = options.split(QLatin1Char(','));
-        }
-
-        // Resolve gvfs mountpoints
-        if (mp->d->m_mountedFrom == QLatin1String("gvfsd-fuse")) {
-            const QDir gvfsDir(mp->d->m_mountPoint);
-            const QStringList mountDirs = gvfsDir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
-            for (const QString &mountDir : mountDirs) {
-                const QString type = mountDir.section(QLatin1Char(':'), 0, 0);
-                if (type.isEmpty()) {
-                    continue;
+                if (infoNeeded & NeedMountOptions) {
+                    mp->d->m_mountOptions = QFile::decodeName(mnt_fs_get_options(fs)).split(QLatin1Char(','));
                 }
 
-                Ptr gvfsmp(new KMountPoint);
-                gvfsmp->d->m_mountedFrom = mp->d->m_mountedFrom;
-                gvfsmp->d->m_mountPoint = mp->d->m_mountPoint + QLatin1Char('/') + mountDir;
-                gvfsmp->d->m_mountType = type;
-                result.append(gvfsmp);
+                if (infoNeeded & NeedRealDeviceName) {
+                    if (mp->d->m_mountedFrom.startsWith(QLatin1Char('/'))) {
+                        mp->d->m_device = mp->d->m_mountedFrom;
+                    }
+                }
+
+                mp->d->resolveGvfsMountPoints(result);
+
+                mp->d->finalizeCurrentMountPoint(infoNeeded);
+                result.push_back(mp);
             }
+
+            mnt_free_iter(itr);
         }
 
-        mp->d->finalizeCurrentMountPoint(infoNeeded);
-
-        result.append(mp);
+        mnt_free_table(table);
     }
-    ENDMNTENT(mnttab);
 #endif
+
     return result;
 }
 
@@ -411,9 +411,7 @@ KMountPoint::Ptr KMountPoint::List::findByPath(const QString &path) const
 #ifndef Q_OS_WIN
     /* If the path contains symlinks, get the real name */
     QFileInfo fileinfo(path);
-    const QString realname = fileinfo.exists()
-                             ? fileinfo.canonicalFilePath()
-                             : fileinfo.absolutePath(); //canonicalFilePath won't work unless file exists
+    const QString realname = fileinfo.exists() ? fileinfo.canonicalFilePath() : fileinfo.absolutePath(); // canonicalFilePath won't work unless file exists
 #else
     const QString realname = QDir::fromNativeSeparators(QDir(path).absolutePath());
 #endif
@@ -439,8 +437,7 @@ KMountPoint::Ptr KMountPoint::List::findByDevice(const QString &device) const
         return Ptr();
     }
     for (const KMountPoint::Ptr &mountPoint : *this) {
-        if (realDevice.compare(mountPoint->d->m_device, cs) == 0 ||
-                realDevice.compare(mountPoint->d->m_mountedFrom, cs) == 0) {
+        if (realDevice.compare(mountPoint->d->m_device, cs) == 0 || realDevice.compare(mountPoint->d->m_mountedFrom, cs) == 0) {
             return mountPoint;
         }
     }
@@ -449,6 +446,7 @@ KMountPoint::Ptr KMountPoint::List::findByDevice(const QString &device) const
 
 bool KMountPoint::probablySlow() const
 {
+    /* clang-format off */
     return d->m_mountType == QLatin1String("nfs")
         || d->m_mountType == QLatin1String("nfs4")
         || d->m_mountType == QLatin1String("cifs")
@@ -457,20 +455,28 @@ bool KMountPoint::probablySlow() const
         // Technically KIOFUSe mounts local slaves as well,
         // such as recents:/, but better safe than sorry...
         || d->m_mountType == QLatin1String("fuse.kio-fuse");
+    /* clang-format on */
 }
 
 bool KMountPoint::testFileSystemFlag(FileSystemFlag flag) const
 {
-    const bool isMsDos = (d->m_mountType == QLatin1String("msdos") || d->m_mountType == QLatin1String("fat") || d->m_mountType == QLatin1String("vfat"));
-    const bool isNtfs = d->m_mountType.contains(QLatin1String("fuse.ntfs")) || d->m_mountType.contains(QLatin1String("fuseblk.ntfs"))
+    /* clang-format off */
+    const bool isMsDos = d->m_mountType == QLatin1String("msdos")
+                         || d->m_mountType == QLatin1String("fat")
+                         || d->m_mountType == QLatin1String("vfat");
+
+    const bool isNtfs = d->m_mountType.contains(QLatin1String("fuse.ntfs"))
+                        || d->m_mountType.contains(QLatin1String("fuseblk.ntfs"))
                         // fuseblk could really be anything. But its most common use is for NTFS mounts, these days.
                         || d->m_mountType == QLatin1String("fuseblk");
-    const bool isSmb = d->m_mountType == QLatin1String("cifs")
-                    || d->m_mountType == QLatin1String("smbfs")
-                    // gvfs-fuse mounted SMB share
-                    || d->m_mountType == QLatin1String("smb-share");
 
-    switch (flag)  {
+    const bool isSmb = d->m_mountType == QLatin1String("cifs")
+                       || d->m_mountType == QLatin1String("smbfs")
+                       // gvfs-fuse mounted SMB share
+                       || d->m_mountType == QLatin1String("smb-share");
+    /* clang-format on */
+
+    switch (flag) {
     case SupportsChmod:
     case SupportsChown:
     case SupportsUTime:
@@ -481,4 +487,3 @@ bool KMountPoint::testFileSystemFlag(FileSystemFlag flag) const
     }
     return false;
 }
-
