@@ -6,15 +6,15 @@
 */
 
 #include "openurljob.h"
-#include "openwithhandlerinterface.h"
-#include "openorexecutefileinterface.h"
-#include "global.h"
-#include "job.h" // for buildErrorString
 #include "commandlauncherjob.h"
 #include "desktopexecparser.h"
-#include "untrustedprogramhandlerinterface.h"
+#include "global.h"
+#include "job.h" // for buildErrorString
 #include "jobuidelegatefactory.h"
 #include "kiogui_debug.h"
+#include "openorexecutefileinterface.h"
+#include "openwithhandlerinterface.h"
+#include "untrustedprogramhandlerinterface.h"
 
 #include <KApplicationTrader>
 #include <KAuthorized>
@@ -30,15 +30,14 @@
 #include <QHostInfo>
 #include <QMimeDatabase>
 #include <QOperatingSystemVersion>
-#include <QTimer>
-
-#include <kio/scheduler.h>
+#include <mimetypefinderjob.h>
 
 class KIO::OpenUrlJobPrivate
 {
 public:
     explicit OpenUrlJobPrivate(const QUrl &url, OpenUrlJob *qq)
-        : m_url(url), q(qq)
+        : m_url(url)
+        , q(qq)
     {
         q->setCapabilities(KJob::Killable);
     }
@@ -48,12 +47,9 @@ public:
     QString externalBrowser() const;
     bool runExternalBrowser(const QString &exe);
     void useSchemeHandler();
-    void determineLocalMimeType();
-    void statFile();
-    void scanFileWithGet();
 
     QUrl m_url;
-    KIO::OpenUrlJob * const q;
+    KIO::OpenUrlJob *const q;
     QString m_suggestedFileName;
     QByteArray m_startupId;
     QString m_mimeTypeName;
@@ -85,12 +81,14 @@ private:
 };
 
 KIO::OpenUrlJob::OpenUrlJob(const QUrl &url, QObject *parent)
-    : KCompositeJob(parent), d(new OpenUrlJobPrivate(url, this))
+    : KCompositeJob(parent)
+    , d(new OpenUrlJobPrivate(url, this))
 {
 }
 
 KIO::OpenUrlJob::OpenUrlJob(const QUrl &url, const QString &mimeType, QObject *parent)
-    : KCompositeJob(parent), d(new OpenUrlJobPrivate(url, this))
+    : KCompositeJob(parent)
+    , d(new OpenUrlJobPrivate(url, this))
 {
     d->m_mimeTypeName = mimeType;
 }
@@ -136,8 +134,7 @@ void KIO::OpenUrlJob::setFollowRedirections(bool b)
 
 static bool checkNeedPortalSupport()
 {
-    return !(QStandardPaths::locate(QStandardPaths::RuntimeLocation, QLatin1String("flatpak-info")).isEmpty()
-            || qEnvironmentVariableIsSet("SNAP"));
+    return !(QStandardPaths::locate(QStandardPaths::RuntimeLocation, QLatin1String("flatpak-info")).isEmpty() || qEnvironmentVariableIsSet("SNAP"));
 }
 
 void KIO::OpenUrlJob::start()
@@ -182,14 +179,22 @@ void KIO::OpenUrlJob::start()
         return;
     }
 
-    if (!KProtocolManager::supportsListing(d->m_url)) {
-        // No support for listing => it can't be a directory (example: http)
-        d->scanFileWithGet();
-        return;
-    }
-
-    // It may be a directory or a file, let's use stat to find out
-    d->statFile();
+    auto *job = new KIO::MimeTypeFinderJob(d->m_url, this);
+    job->setFollowRedirections(d->m_followRedirections);
+    job->setSuggestedFileName(d->m_suggestedFileName);
+    connect(job, &KJob::result, this, [job, this]() {
+        const int errCode = job->error();
+        if (errCode) {
+            setError(errCode);
+            setErrorText(job->errorText());
+            emitResult();
+        } else {
+            d->m_suggestedFileName = job->suggestedFileName();
+            d->m_mimeTypeName = job->mimeType();
+            d->runUrlWithMimeType();
+        }
+    });
+    job->start();
 }
 
 bool KIO::OpenUrlJob::doKill()
@@ -241,7 +246,7 @@ bool KIO::OpenUrlJobPrivate::runExternalBrowser(const QString &exec)
 void KIO::OpenUrlJobPrivate::useSchemeHandler()
 {
     // look for an application associated with x-scheme-handler/<protocol>
-   const KService::Ptr service = KApplicationTrader::preferredService(QLatin1String("x-scheme-handler/") + m_url.scheme());
+    const KService::Ptr service = KApplicationTrader::preferredService(QLatin1String("x-scheme-handler/") + m_url.scheme());
     if (service) {
         startService(service);
         return;
@@ -259,55 +264,6 @@ void KIO::OpenUrlJobPrivate::useSchemeHandler()
     }
 }
 
-void KIO::OpenUrlJobPrivate::statFile()
-{
-    Q_ASSERT(m_mimeTypeName.isEmpty());
-
-    KIO::StatJob *job = KIO::statDetails(m_url, KIO::StatJob::SourceSide, KIO::StatBasic, KIO::HideProgressInfo);
-    job->setUiDelegate(nullptr);
-    QObject::connect(job, &KJob::result,
-                     q, [=]() {
-        const int errCode = job->error();
-        if (errCode) {
-            // ERR_NO_CONTENT is not an error, but an indication no further
-            // actions needs to be taken.
-            if (errCode != KIO::ERR_NO_CONTENT) {
-                q->setError(errCode);
-                q->setErrorText(KIO::buildErrorString(errCode, job->errorText()));
-            }
-            q->emitResult();
-            return;
-        }
-        if (m_followRedirections) { // Update our URL in case of a redirection
-            m_url = job->url();
-        }
-
-        const KIO::UDSEntry entry = job->statResult();
-
-        const QString localPath = entry.stringValue(KIO::UDSEntry::UDS_LOCAL_PATH);
-        if (!localPath.isEmpty()) {
-            m_url = QUrl::fromLocalFile(localPath);
-        }
-
-        // MIME type already known? (e.g. print:/manager)
-        m_mimeTypeName = entry.stringValue(KIO::UDSEntry::UDS_MIME_TYPE);
-        if (!m_mimeTypeName.isEmpty()) {
-            runUrlWithMimeType();
-            return;
-        }
-
-        if (entry.isDir()) {
-            m_mimeTypeName = QStringLiteral("inode/directory");
-            runUrlWithMimeType();
-        } else { // it's a file
-            // Start the timer. Once we get the timer event this
-            // protocol server is back in the pool and we can reuse it.
-            // This gives better performance than starting a new slave
-            QTimer::singleShot(0, q, [this] { scanFileWithGet(); });
-        }
-    });
-}
-
 void KIO::OpenUrlJobPrivate::startService(const KService::Ptr &service, const QList<QUrl> &urls)
 {
     KIO::ApplicationLauncherJob *job = new KIO::ApplicationLauncherJob(service, q);
@@ -315,94 +271,10 @@ void KIO::OpenUrlJobPrivate::startService(const KService::Ptr &service, const QL
     job->setRunFlags(m_deleteTemporaryFile ? KIO::ApplicationLauncherJob::DeleteTemporaryFiles : KIO::ApplicationLauncherJob::RunFlags{});
     job->setSuggestedFileName(m_suggestedFileName);
     job->setStartupId(m_startupId);
-    job->setUiDelegate(q->uiDelegate());
     q->addSubjob(job);
     job->start();
 }
 
-static QMimeType fixupMimeType(const QString &mimeType, const QString &fileName)
-{
-    QMimeDatabase db;
-    QMimeType mime = db.mimeTypeForName(mimeType);
-    if ((!mime.isValid() || mime.isDefault()) && !fileName.isEmpty()) {
-        mime = db.mimeTypeForFile(fileName, QMimeDatabase::MatchExtension);
-    }
-    return mime;
-}
-
-void KIO::OpenUrlJobPrivate::scanFileWithGet()
-{
-    Q_ASSERT(m_mimeTypeName.isEmpty());
-
-    // First, let's check for well-known extensions
-    // Not over HTTP and not when there is a query in the URL, in any case.
-    if (!m_url.hasQuery() && !m_url.scheme().startsWith(QLatin1String("http"))) {
-        QMimeDatabase db;
-        QMimeType mime = db.mimeTypeForUrl(m_url);
-        if (!mime.isDefault()) {
-            //qDebug() << "Scanfile: MIME TYPE is " << mime.name();
-            m_mimeTypeName = mime.name();
-            runUrlWithMimeType();
-            return;
-        }
-    }
-
-    // No MIME type found, and the URL is not local  (or fast mode not allowed).
-    // We need to apply the 'KIO' method, i.e. either asking the server or
-    // getting some data out of the file, to know what MIME type it is.
-
-    if (!KProtocolManager::supportsReading(m_url)) {
-        qCWarning(KIO_GUI) << "#### NO SUPPORT FOR READING!";
-        q->setError(KIO::ERR_CANNOT_READ);
-        q->setErrorText(m_url.toDisplayString());
-        q->emitResult();
-        return;
-    }
-    //qDebug() << this << "Scanning file" << url;
-
-    KIO::TransferJob *job = KIO::get(m_url, KIO::NoReload /*reload*/, KIO::HideProgressInfo);
-    job->setUiDelegate(nullptr);
-    QObject::connect(job, &KJob::result, q, [=]() {
-        const int errCode = job->error();
-        if (errCode) {
-             // ERR_NO_CONTENT is not an error, but an indication no further
-             // actions needs to be taken.
-             if (errCode != KIO::ERR_NO_CONTENT) {
-                 q->setError(errCode);
-                 q->setErrorText(job->errorText());
-             }
-             q->emitResult();
-        }
-        // if the job succeeded, we certainly hope it emitted mimetype()...
-    });
-    QObject::connect(job, &KIO::TransferJob::mimeTypeFound,
-                     q, [=](KIO::Job *, const QString &mimetype) {
-        if (m_followRedirections) { // Update our URL in case of a redirection
-            m_url = job->url();
-        }
-        if (mimetype.isEmpty()) {
-            qCWarning(KIO_GUI) << "get() didn't emit a MIME type! Probably a kioslave bug, please check the implementation of" << m_url.scheme();
-        }
-        m_mimeTypeName = mimetype;
-
-        // If the current MIME type is the default MIME type, then attempt to
-        // determine the "real" MIME type from the file name (bug #279675)
-        const QMimeType mime = fixupMimeType(m_mimeTypeName, m_suggestedFileName.isEmpty() ? m_url.fileName() : m_suggestedFileName);
-        const QString mimeName = mime.name();
-        if (mime.isValid() && mimeName != m_mimeTypeName) {
-            m_mimeTypeName = mimeName;
-        }
-
-        if (m_suggestedFileName.isEmpty()) {
-            m_suggestedFileName = job->queryMetaData(QStringLiteral("content-disposition-filename"));
-        }
-
-        job->putOnHold();
-        KIO::Scheduler::publishSlaveOnHold();
-        runUrlWithMimeType();
-    });
-}
-
 void KIO::OpenUrlJobPrivate::runLink(const QString &filePath, const QString &urlStr, const QString &optionalServiceName)
 {
     if (urlStr.isEmpty()) {
@@ -445,8 +317,7 @@ static bool isBinary(const QMimeType &mimeType)
     // - MIME types that inherit application/x-executable _and_ text/plain are scripts, these are
     //   handled by handleScripts()
 
-    return (mimeType.inherits(QStringLiteral("application/x-executable"))
-            || mimeType.inherits(QStringLiteral("application/x-sharedlib"))
+    return (mimeType.inherits(QStringLiteral("application/x-executable")) || mimeType.inherits(QStringLiteral("application/x-sharedlib"))
             || mimeType.inherits(QStringLiteral("application/x-ms-dos-executable")));
 }
 
@@ -454,8 +325,7 @@ static bool isBinary(const QMimeType &mimeType)
 // e.g. ".sh", ".csh", ".py", ".js"
 static bool isTextScript(const QMimeType &mimeType)
 {
-    return (mimeType.inherits(QStringLiteral("application/x-executable"))
-            && mimeType.inherits(QStringLiteral("text/plain")));
+    return (mimeType.inherits(QStringLiteral("application/x-executable")) && mimeType.inherits(QStringLiteral("text/plain")));
 }
 
 // Helper function that returns whether a file has the execute bit set or not.
@@ -476,8 +346,10 @@ void KIO::OpenUrlJobPrivate::handleBinaries(const QMimeType &mimeType)
     // Don't run remote executables
     if (!isLocal) {
         q->setError(KJob::UserDefinedError);
-        q->setErrorText(i18n("The executable file \"%1\" is located on a remote filesystem. "
-                             "For safety reasons it will not be started.", m_url.toDisplayString()));
+        q->setErrorText(
+            i18n("The executable file \"%1\" is located on a remote filesystem. "
+                 "For safety reasons it will not be started.",
+                 m_url.toDisplayString()));
         q->emitResult();
         return;
     }
@@ -509,14 +381,6 @@ void KIO::OpenUrlJobPrivate::handleBinaries(const QMimeType &mimeType)
 
 void KIO::OpenUrlJobPrivate::handleBinariesHelper(const QString &localPath, bool isNativeBinary)
 {
-    // For local .exe files, open in the default app (e.g. WINE)
-    if (!isNativeBinary) {
-        openInPreferredApp();
-        return;
-    }
-
-    // Native binaries
-
     if (!m_runExecutables) {
         q->setError(KJob::UserDefinedError);
         q->setErrorText(i18n("For security reasons, launching executables is not allowed in this context."));
@@ -524,6 +388,13 @@ void KIO::OpenUrlJobPrivate::handleBinariesHelper(const QString &localPath, bool
         return;
     }
 
+    // For local .exe files, open in the default app (e.g. WINE)
+    if (!isNativeBinary) {
+        openInPreferredApp();
+        return;
+    }
+
+    // Native binaries
     if (!hasExecuteBit(localPath)) {
         // Show untrustedProgram dialog for local, native executables without the execute bit
         showUntrustedProgramWarningDialog(localPath);
@@ -568,7 +439,6 @@ void KIO::OpenUrlJobPrivate::executeCommand()
 {
     // Execute the URL as a command. This is how we start scripts and executables
     KIO::CommandLauncherJob *job = new KIO::CommandLauncherJob(m_url.toLocalFile(), QStringList());
-    job->setUiDelegate(q->uiDelegate());
     job->setStartupId(m_startupId);
     job->setWorkingDirectory(m_url.adjusted(QUrl::RemoveFilename).toLocalFile());
     q->addSubjob(job);
@@ -594,7 +464,6 @@ void KIO::OpenUrlJobPrivate::runUrlWithMimeType()
         return;
     }
 
-
     // Scripts and executables
     QMimeDatabase db;
     const QMimeType mimeType = db.mimeTypeForName(m_mimeTypeName);
@@ -629,8 +498,8 @@ void KIO::OpenUrlJobPrivate::handleDesktopFiles()
         return;
     }
 
-    if (m_url.fileName() == QLatin1String(".directory")) {
-        // We cannot execute a .directory file, open in the default app
+    if (m_url.fileName() == QLatin1String(".directory") || m_mimeTypeName == QLatin1String("application/x-theme")) {
+        // We cannot execute these files, open in the default app
         m_mimeTypeName = QStringLiteral("text/plain");
         openInPreferredApp();
         return;
@@ -772,8 +641,7 @@ void KIO::OpenUrlJobPrivate::showOpenOrExecuteFileDialog(std::function<void(bool
     auto *openOrExecuteFileHandler = KIO::delegateExtension<KIO::OpenOrExecuteFileInterface *>(q);
     if (!openOrExecuteFileHandler) {
         // No way to ask the user whether to execute or open
-        if (isTextScript(mimeType)
-            || mimeType.inherits(QStringLiteral("application/x-desktop"))) { // Open text-based ones in the default app
+        if (isTextScript(mimeType) || mimeType.inherits(QStringLiteral("application/x-desktop"))) { // Open text-based ones in the default app
             openInPreferredApp();
         } else {
             q->setError(KJob::UserDefinedError);
@@ -788,8 +656,7 @@ void KIO::OpenUrlJobPrivate::showOpenOrExecuteFileDialog(std::function<void(bool
         q->emitResult();
     });
 
-    QObject::connect(openOrExecuteFileHandler, &KIO::OpenOrExecuteFileInterface::executeFile,
-                     q, [this, dialogFinished](bool shouldExecute) {
+    QObject::connect(openOrExecuteFileHandler, &KIO::OpenOrExecuteFileInterface::executeFile, q, [this, dialogFinished](bool shouldExecute) {
         m_runExecutables = shouldExecute;
         dialogFinished(shouldExecute);
     });
@@ -803,6 +670,7 @@ void KIO::OpenUrlJob::slotResult(KJob *job)
     const int errCode = job->error();
     if (errCode) {
         setError(errCode);
+        // We're a KJob, not a KIO::Job, so build the error string here
         setErrorText(KIO::buildErrorString(errCode, job->errorText()));
     }
     emitResult();
