@@ -32,6 +32,8 @@ class KIOWIDGETS_EXPORT PreviewJob : public KIO::Job
 public:
     /**
      * Specifies the type of scaling that is applied to the generated preview.
+     * For HiDPI, pixel density scaling, @see setDevicePixelRatio
+     *
      * @since 4.7
      */
     enum ScaleType {
@@ -50,7 +52,7 @@ public:
          * the PreviewJob. The result will be cached for later use. Per default
          * ScaledAndCached is set.
          */
-        ScaledAndCached
+        ScaledAndCached,
     };
 
 #if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(4, 7)
@@ -75,9 +77,7 @@ public:
      *             \p enabledPlugins has been slightly changed.
      */
     KIOWIDGETS_DEPRECATED_VERSION(4, 7, "Use PreviewJob(const KFileItemList&, const QSize&, const QStringList*)")
-    PreviewJob(const KFileItemList &items, int width, int height,
-               int iconSize, int iconAlpha, bool scale, bool save,
-               const QStringList *enabledPlugins);
+    PreviewJob(const KFileItemList &items, int width, int height, int iconSize, int iconAlpha, bool scale, bool save, const QStringList *enabledPlugins);
 #endif
 
     /**
@@ -89,9 +89,7 @@ public:
      *                       KConfigGroup "PreviewSettings" are used.
      * @since 4.7
      */
-    PreviewJob(const KFileItemList &items,
-               const QSize &size,
-               const QStringList *enabledPlugins = nullptr);
+    PreviewJob(const KFileItemList &items, const QSize &size, const QStringList *enabledPlugins = nullptr);
 
     virtual ~PreviewJob();
 
@@ -174,6 +172,46 @@ public:
      **/
     int sequenceIndex() const;
 
+    /**
+     * Returns the index at which the thumbs of a ThumbSequenceCreator start
+     * wrapping around ("looping"). Fractional values may be returned if the
+     * ThumbSequenceCreator supports sub-integer precision, but frontends
+     * supporting only integer sequence indices may choose to round it down.
+     *
+     * @see ThumbSequenceCreator::sequenceIndexWraparoundPoint()
+     * @since 5.80
+     */
+    float sequenceIndexWraparoundPoint() const;
+
+    /**
+     * Determines whether the ThumbCreator in use is a ThumbSequenceCreator.
+     *
+     * @since 5.80
+     */
+    bool handlesSequences() const;
+
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 84)
+    /**
+     * Request preview to use the device pixel ratio @p dpr.
+     * The returned thumbnail may not respect the device pixel ratio requested.
+     * Use QPixmap::devicePixelRatio to check, or paint as necessary.
+     *
+     * @since 5.80
+     * @deprecated Since 5.84, use setDevicePixelRatio(qreal dpr) instead
+     */
+    KIOWIDGETS_DEPRECATED_VERSION(5, 84, "Use setDevicePixelRatio(qreal dpr)")
+    void setDevicePixelRatio(int dpr);
+#endif
+
+    /**
+     * Request preview to use the device pixel ratio @p dpr.
+     * The returned thumbnail may not respect the device pixel ratio requested.
+     * Use QPixmap::devicePixelRatio to check, or paint as necessary.
+     *
+     * @since 5.84
+     */
+    void setDevicePixelRatio(qreal dpr);
+
     /**
      * Returns a list of all available preview plugins. The list
      * contains the basenames of the plugins' .desktop files (no path,
@@ -232,6 +270,31 @@ private:
     Q_PRIVATE_SLOT(d_func(), void startPreview())
     Q_PRIVATE_SLOT(d_func(), void slotThumbData(KIO::Job *, const QByteArray &))
     Q_DECLARE_PRIVATE(PreviewJob)
+
+public:
+    /**
+     * Sets a default device Pixel Ratio used for Previews
+     * @see setDevicePixelRatio
+     *
+     * Defaults to 1
+     *
+     * @since 5.80
+     */
+    static void setDefaultDevicePixelRatio(int devicePixelRatio);
+
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 84)
+    /**
+     * Sets a default device Pixel Ratio used for Previews
+     * @see setDevicePixelRatio
+     *
+     * Defaults to 1
+     *
+     * @since 5.84
+     * @deprecated Since 5.84, use setDefaultDevicePixelRatio(qreal dpr) instead
+     */
+    KIOWIDGETS_DEPRECATED_VERSION(5, 84, "Use setDefaultDevicePixelRatio(qreal dpr)")
+    static void setDefaultDevicePixelRatio(qreal devicePixelRatio);
+#endif
 };
 
 #if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(4, 7)
@@ -258,8 +321,16 @@ private:
  *             with the setter-methods instead. Note that the semantics of
  *             \p enabledPlugins has been slightly changed.
  */
+KIOWIDGETS_EXPORT
 KIOWIDGETS_DEPRECATED_VERSION(4, 7, "Use KIO::filePreview(const KFileItemList &, const QSize &, const QStringList *")
-KIOWIDGETS_EXPORT PreviewJob *filePreview(const KFileItemList &items, int width, int height = 0, int iconSize = 0, int iconAlpha = 70, bool scale = true, bool save = true, const QStringList *enabledPlugins = nullptr);   // KDE5: use enums instead of bool scale + bool save
+PreviewJob *filePreview(const KFileItemList &items,
+                        int width,
+                        int height = 0,
+                        int iconSize = 0,
+                        int iconAlpha = 70,
+                        bool scale = true,
+                        bool save = true,
+                        const QStringList *enabledPlugins = nullptr); // KDE5: use enums instead of bool scale + bool save
 #endif
 
 #if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(4, 7)
@@ -286,8 +357,16 @@ KIOWIDGETS_EXPORT PreviewJob *filePreview(const KFileItemList &items, int width,
  *             with the setter-methods instead. Note that the semantics of
  *             \p enabledPlugins has been slightly changed.
  */
+KIOWIDGETS_EXPORT
 KIOWIDGETS_DEPRECATED_VERSION(4, 7, "Use KIO::filePreview(const KFileItemList &, const QSize &, const QStringList *")
-KIOWIDGETS_EXPORT PreviewJob *filePreview(const QList<QUrl> &items, int width, int height = 0, int iconSize = 0, int iconAlpha = 70, bool scale = true, bool save = true, const QStringList *enabledPlugins = nullptr);
+PreviewJob *filePreview(const QList<QUrl> &items,
+                        int width,
+                        int height = 0,
+                        int iconSize = 0,
+                        int iconAlpha = 70,
+                        bool scale = true,
+                        bool save = true,
+                        const QStringList *enabledPlugins = nullptr);
 #endif
 
 /**
